<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

set_include_path(dirname(__FILE__) . '/hessian/');
require_once(dirname(__FILE__) . '/hessian/HessianClient.php');

class WoogleIndexServiceRemote extends WoogleIndexService {
	protected $proxyIndex;
	private $reIndexCount = 0;
	
	public function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		
		$this->proxyIndex = &new HessianClient($this->getLocation(), WoogleConfig::get('remoteOptions'));
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function checkIndexService(){
		WoogleUtil::profileIn(__METHOD__);
		$r = false;
		$r = $this->proxyIndex->check($this->getIndexAuthKey(), WoogleConfig::get('indexRepository'));
		return WoogleUtil::profileOut(__METHOD__, $r);
	}
	
	public function getLocation() {
		WoogleUtil::profileIn(__METHOD__);
		$serviceUrl = WoogleConfig::get('remoteIndexServiceUrl');
		if ($serviceUrl == '') $serviceUrl =  WoogleConfig::get('remoteBaseServiceUrl') . 'phpIndex';
		return WoogleUtil::profileOut(__METHOD__, $serviceUrl);
	}
	
	
	public function numDocs() {
		WoogleUtil::profileIn(__METHOD__);
		//TODO implement
		return WoogleUtil::profileOut(__METHOD__, -1);
	}
	
	public function exists($id) {
		WoogleUtil::profileIn(__METHOD__);
		$exists = $this->proxyIndex->exists($this->getIndexAuthKey(), WoogleQueryServiceCommon::getId($id));
		WoogleUtil::profileOut(__METHOD__);
		return $exists;
	}
	
	public function getIndexFileSize() {
		return -1;
	}
	
	protected function startReIndex(){
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::info('Starting ReIndex using RemoteQS');
		$this->reIndexCount = 0;
		$this->proxyIndex->startCompleteReIndex($this->getIndexAuthKey(), WoogleConfig::get('indexRepository'));
		WoogleUtil::profileOut(__METHOD__);
	}
	
	protected function stopReIndex(){
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::info('Stoppping ReIndex using RemoteQS - added ' . $this->reIndexCount . ' documents');
		$this->proxyIndex->stopCompleteReIndex($this->getIndexAuthKey(), WoogleConfig::get('indexRepository'));
		$this->optimizeIndex();
		$this->reIndexCount = 0;
		WoogleUtil::profileOut(__METHOD__);
	}
	
	protected function optimizeIndex(){
		WoogleUtil::profileIn(__METHOD__);
		$this->proxyIndex->optimize($this->getIndexAuthKey(), WoogleConfig::get('indexRepository'));
		WoogleUtil::profileOut(__METHOD__);
	}

	
	public function doAddItem($wd, $flush=true) {
		WoogleUtil::profileIn(__METHOD__);
		try{
			$this->proxyIndex->addIndexable($this->getIndexAuthKey(), $this->createKA($wd));
		} catch (Exception $e) {
			WoogleUtil::error("Can not add article " . $wd->getUri() . " to remote index: " . $e);
		}
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function doAddItems($wda, $flush=true, $optimize=false, $printStatus=false){
		WoogleUtil::profileIn(__METHOD__);

		if ($printStatus) echo "Writing " . count($wda) . " pages to the index";
		// convert articles to knowledge artifacts
		$kalist = array();
		
		foreach($wda as $wd){
			try{
				if ($printStatus) echo ".";
				$this->doAddItem($wd);
				$this->reIndexCount++;
				//array_push($kalist, $this->createKA($article));
			} catch (Exception $e) {
				WoogleUtil::error("Can not add article " . $wd->getUri() . " to remote index: " . $e);
			}
		}
			
		/*
		// send to remote index service
		if ($flush){
			$this->proxyIndex->addIndexablesAndOptimize($this->getIndexAuthKey(), $kalist);
		} else {
			$this->proxyIndex->addIndexables($this->getIndexAuthKey(), $kalist);
		}*/
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function doUpdateItem($wd) {
		WoogleUtil::profileIn(__METHOD__);
		$this->proxyIndex->updateIndexable($this->getIndexAuthKey(), $this->createKA($wd));
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public function doDeleteItem($id) {
		WoogleUtil::profileIn(__METHOD__);
		$ka = array();
		$ka[WoogleFd::$id] 			= WoogleQueryServiceCommon::getId($id);
		$ka[WoogleFd::$repository] 	= WoogleQueryServiceCommon::getRepository();
		$ka[WoogleFd::$mimetype] 	= WoogleQueryServiceCommon::getMimetype();
		$ka[WoogleFd::$source] 		= WoogleQueryServiceCommon::$source;
		$ka[WoogleFd::$scope] 		= WoogleQueryServiceCommon::$scope;
		$ka[WoogleFd::$genre] 		= WoogleQueryServiceCommon::$genre;
		$ka[WoogleFd::$language] 	= WoogleQueryServiceCommon::$language;
		
		$this->proxyIndex->deleteIndexable($this->getIndexAuthKey(), $ka);
		//$this->proxyIndex->flush($this->getIndexAuthKey());
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	private function getIndexAuthKey() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, WoogleConfig::get('remotePushIndexAuthKey'));
	}
	
	private function createKA($wd) {
		WoogleUtil::profileIn(__METHOD__);
		
		$ka = array();
		
//		echo $article->getTitle()->getFullText() . ': ' . $article->getTitle()->getArticleID() . "\n";
//		$doc->addField(Zend_Search_Lucene_Field::Keyword(WoogleFd::$uri, WoogleMWUtil::getArticleAbsolutePath() . $tt->getPartialURL()));
		
		$ka[WoogleFd::$id] 			= $wd->getId();
		$ka[WoogleFd::$uri] 		= utf8_decode($wd->getUri());
		$ka[WoogleFd::$label] 		= utf8_decode($wd->getLabel());
		$ka[WoogleFd::$content] 	= utf8_decode(substr($wd->getContent(), 0, min(strlen($wd->getContent()), 50000))); // TODO quick hack for http://octopus13.fzi.de:8080/browse/WOO-176
		$ka[WoogleFd::$mimetype] 	= $wd->getMimetype();
		$ka[WoogleFd::$created] 	= WoogleMWUtil::getDateTime($wd->getCreated());  // MediaWiki concatenated string timestamp (YYYYMMDDHHMMSS) 
		$ka[WoogleFd::$changed] 	= WoogleMWUtil::getDateTime($wd->getChanged()); 
		$ka[WoogleFd::$source] 		= $wd->getSource();
		$ka[WoogleFd::$repository] 	= $wd->getRepository();
		$ka[WoogleFd::$pages] 		= $wd->getPages();
		$ka[WoogleFd::$size] 		= $wd->getSize();
		$ka[WoogleFd::$scope] 		= $wd->getScope();
		$ka[WoogleFd::$genre] 		= $wd->getGenre();
		$ka[WoogleFd::$language] 	= $wd->getLanguage();
		
		// Debug
		
		WoogleUtil::debug('Remotely storing label: ' 		. $wd->getLabel());
		WoogleUtil::debug('Remotely storing url: ' 			. $wd->getUri());
		
		$cx = substr($wd->getContent(), 0, min(strlen($wd->getContent()), 50000));
		WoogleUtil::debug('Remotely storing content1: ' 	. $cx);
		WoogleUtil::debug('Remotely storing content2: ' 	. utf8_decode($cx));
		WoogleUtil::debug('Remotely storing content3: ' 	. utf8_encode($cx));
		
		WoogleUtil::profileOut(__METHOD__);
		return $ka;
	}
	
	
}

?>