<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 


set_include_path(dirname(__FILE__) . '/hessian/');
require_once(dirname(__FILE__) . '/hessian/HessianClient.php');

class WoogleQueryManagerRemote implements WoogleQueryManager {
	protected $proxy;
	
	public function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		
		$serviceUrl = WoogleConfig::get('remoteQueryManagerUrl');
		if ($serviceUrl == '') $serviceUrl =  WoogleConfig::get('remoteBaseServiceUrl') . 'phpQueryManager';
		$this->proxy = &new HessianClient($serviceUrl, WoogleConfig::get('remoteOptions'));
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * actually not required for the remote backend
	 * (logging is done automatically as part of the query process) 
	 */
	public function handleQuery($searchString, $numHits) {}
	public function handleBrowse($searchString, $sessionHash) {}
	public function handleClick($searchString, $url, $position, $session) {}
	
	/**
	 * @return in teamweaver code: List<QueryMetadata>, to be done
	 */
	public function getUnsatisfiedQueriesWithFewHits($number) {
		WoogleUtil::profileIn(__METHOD__);
		$result = $this->proxy->getUnsatisfiedQueriesWithFewHits(WoogleConfig::get('remoteAuthKey'), $number);
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * @return in teamweaver code: List<QueryMetadata>, to be done
	 */
	public function getUnsatisfiedQueriesWithManyDifferentClicks($number) {
		WoogleUtil::profileIn(__METHOD__);
		$result = $this->proxy->getUnsatisfiedQueriesWithManyDifferentClicks(WoogleConfig::get('remoteAuthKey'), $number);
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * @return in teamweaver code: List<QueryMetadata>, to be done
	 */
	public function getUnsatisfiedQueriesWithMuchBrowsing($number) {
		WoogleUtil::profileIn(__METHOD__);
		$result = $this->proxy->getUnsatisfiedQueriesWithMuchBrowsing(WoogleConfig::get('remoteAuthKey'), $number);
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * @return in teamweaver code: List<InformationNeedSet>, to be done
	 */
	public function getUnsatisfiedQueriesAggregations($number) {
		WoogleUtil::profileIn(__METHOD__);
		$result = $this->proxy->getUnsatisfiedQueriesAggregations(WoogleConfig::get('remoteAuthKey'), $number);
		return WoogleUtil::profileOut(__METHOD__,	$result);
	}
	
	/**
	 * @return in teamweaver code: InformationNeed, to be done
	 */

	public function getInformationNeed($searchString) {
		WoogleUtil::profileIn(__METHOD__);
		$qStart = microtime(true);
		
		$result = $this->proxy->getInformationNeed(WoogleConfig::get('remoteAuthKey'), utf8_decode($searchString));
		
		if ($result === null){
			$result = false; // to harmonize with QueryManagerNative
		} else {
			// adjust date values
			$firstDate 	= $result['firstDate'];
			$lastDate 	= $result['lastDate'];
			$result['firstDate'] 	= $firstDate->getTimestamp();
			$result['lastDate'] 	= $lastDate->getTimestamp();
		}
		
		// do some logging
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		WoogleUtil::perflog($qTime . ';' . 'infneed', 'querymetadata');
		return WoogleUtil::profileOut(__METHOD__, $result);
	}

	/**
	 * @return in teamweaver code: List<QueryMetadata>, to be done
	 */
	public function getRelatedQueries($searchString, $max, $excludeQuery) {
		WoogleUtil::profileIn(__METHOD__);
		$qStart = microtime(true);
		$rqueries = array();
		
		$result = $this->proxy->getRelatedQueries(WoogleConfig::get('remoteAuthKey'), utf8_decode($searchString), $max, $excludeQuery);
		WoogleUtil::debug('Get related queries for ' . utf8_decode($searchString) . ' found '  . print_r($result, true));
		if ($result != ''){ // TODO can this be?
			foreach ($result as $res){
				// adjust date values
				$firstDate 	= $res['firstDate'];
				$lastDate 	= $res['lastDate'];
				$res['firstDate'] 	= $firstDate->getTimestamp();
				$res['lastDate'] 	= $lastDate->getTimestamp();
				
				array_push($rqueries, $res); 
			}
		}
		
		// do some logging
		$qStop = microtime(true);
		$qTime = $qStop - $qStart;
		WoogleUtil::perflog($qTime . ';' . 'related', 'querymetadata');
		return WoogleUtil::profileOut(__METHOD__, $rqueries);
	}
	
	/**
	 * @return array of WoogleTermFreq objects with $number elements
	 */
	public function getPopularQueries($number) {
		WoogleUtil::profileIn(__METHOD__);
		/*
	  	 * @return				array(
		 * 							array(
		 * 								'entity'	=> string,
		 * 								'name'		=> string,
		 * 								'freq'		=> int,
		 * 							),
		 * 							...
		 * 						)
		 */
		$back = $this->proxy->getPopularQueries(WoogleConfig::get('remoteAuthKey'), $number);
		$result = array();
		foreach ($back as $index => $termfreq) {
			$result[$index] = new WoogleTermFreq($termfreq['entity'],$termfreq['freq'],$termfreq['name']);
		}
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * @return array of WoogleEntityFreq objects. the related entities are arrays which describe
	 * @return each of the requested "popular results" respectively (with keys e.g.: id, titleURI, label, created...)
	 * @return in teamweaver these entities are KnowledgeArtifact objects.
	 */
	public function getPopularResults($number) {
		WoogleUtil::profileIn(__METHOD__);
		    /*
			 * @return				array(
			 * 							array(
			 * 								'freq'		=> int,
			 * 								'entity'	=> array(
			 * 									'id'		=> int,
			 * 									'uri'		=> string,
			 * 									'label'		=> string,
			 * 									'content'	=> string,
			 * 									'mimetype'	=> string,
			 * 									'changed'	=> date,
			 * 									'source'	=> string,
			 * 									'size'		=> int,
			 * 								),
			 * 							),
			 * 							...
			 * 						)
			 */
		$back = $this->proxy->getPopularResults(WoogleConfig::get('remoteAuthKey'), $number);
		$result = array();
		foreach ($back as $index => $entfreq) {
			$result[$index] = new WoogleEntityFreq($entfreq['entity'],$entfreq['freq']);
		}
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * @return array of WoogleEntityFreq objects. the related entities are arrays which describe
	 * @return each of the requested "popular sources" respectively (with keys e.g.: repoId, repoName, srcType...)
	 * @return in teamweaver these entities are RepositoryInfo objects.
	 */
	public function getPopularSources($number) {
		WoogleUtil::profileIn(__METHOD__);
		/*
		 * @return				array(
		 * 							array(
		 * 								'freq'		=> int,
		 * 								'entity'	=> array(
		 * 									'repoId'			=> int,
		 * 									'repoName'			=> string,
		 * 									'srcType'			=> string,
		 * 									'srcVersion'		=> string,
		 * 									'connectURL'		=> string,
		 * 									'connectType'		=> string,
		 * 									'user'				=> string,
		 * 									'pass'				=> string,
		 * 									'group'				=> string,
		 * 									'linkPath'			=> string,
		 * 									'contributionURL'	=> string,
		 * 									'incomingFilePath'	=> string,
		 * 									'params'			=> array(string => string, ...),
		 * 									'MimeTypes2Add'		=> array(string, ...),
		 * 									'MimeTypes2Remove'	=> array(string, ...),
		 * 									'maxSizeBytes'		=> int,
		 * 									'connectionTest'	=> bool,
		 * 								),
		 * 							),
		 * 							...
		 * 						)
		 */
		$back = $this->proxy->getPopularSources(WoogleConfig::get('remoteAuthKey'), $number);
		$result = array();
		foreach ($back as $index => $entfreq) {
			$result[$index] = new WoogleEntityFreq($entfreq['entity'],$entfreq['freq']);
		}
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * @return array of WoogleTermFreq objects
	 */
	public function getTags($uri) {
		WoogleUtil::profileIn(__METHOD__);
		$back = $this->proxy->getTags(WoogleConfig::get('remoteAuthKey'), $uri);
		$result = array();
		foreach ($back as $index => $termfreq) {
			$result[$index] = new WoogleTermFreq($termfreq['entity'],$termfreq['freq'],$termfreq['name']);
		}
		return WoogleUtil::profileOut(__METHOD__, $result);
	}
	
	/**
	 * not yet accessible in the backend interface
	 *
	 * @return in teamweaver code: ?, to be done
	 */
	public function getAvgPages() {
		WoogleUtil::profileIn(__METHOD__);
		// $result = $this->proxy->getAvgPages();
		return WoogleUtil::profileOut(__METHOD__, 1); // TODO dummy
	}
	
	/**
	 * not yet accessible in the backend interface
	 *
	 * @return in teamweaver code: ?, to be done
	 */
	public function getAvgRecency() {
		WoogleUtil::profileIn(__METHOD__);
		// $result = $this->proxy->getAvgRecency();
		return WoogleUtil::profileOut(__METHOD__, 0); // TODO dummy
	}
}
?>