<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

abstract class WoogleJob extends Job {
	public static $timeoutCommand = 'woogleTimeout';
	
	protected $errorMsg = '';
	
	public function __construct($command, $title, $params, $id = 0) {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::debug('Creating job: ' . $command . " / Params: " .  str_replace(chr(10), '', print_r($params, true)));
		parent::__construct($command, $title, $params, $id);
		WoogleUtil::debug('Created job' . $this->getLastError());
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * This is the method called for the execution of all Woogle jobs (excluding WoogleJobTimeout)
	 * 
	 * @return 		boolean, if job execution was successfull
	 */
	final public function run() {
		WoogleUtil::profileIn(__METHOD__);
		$maxTimestamp = 0;
		$numJobs = 0;
		$this->errorMsg = '';
		
		WoogleUtil::debug('Executing job: ' . $this->title . " / Params: " . print_r($this->params, true));
		
		// remove all timeout jobs from the queue and save the most recent timeout value
		while ($timeoutJob = Job::pop_type(self::$timeoutCommand)) {
			$numJobs++;
			if ($timeoutJob->params['timestamp'] > $maxTimestamp) {
				$maxTimestamp = $timeoutJob->params['timestamp'];
			}
		}
		
		if ($maxTimestamp < (time() - WoogleConfig::get('timeout'))) {
			// configured timeout has expired -> try again
			
			if ($numJobs > 0){
				WoogleUtil::debug("Timeout expired or not existing - trying to execute job... max timeout is: " . $maxTimestamp . ", number of timeout jobs: " . $numJobs);
			} else {
				WoogleUtil::debug("Timeout not existing - trying to execute job...");
			}		
			
			// execute current job
			if ($this->runJob()) {
				// success
				WoogleUtil::debug("WoogleJob executed successfully");
				if ($numJobs > 0){
					WoogleUtil::error("No more timeout when executing job");
					WoogleUtil::sendErrorMail("No more timeout when executing job", "WoogleJob executed successfully after timeout");
				}
				return WoogleUtil::profileOut(__METHOD__, true);
			} else {
				// job did not execute properly (i.e. threw timeout exception)
				// -> enqueue and add new timeout job with current time
				WoogleUtil::error("WoogleJob failed - adding new timeout");
				if ($numJobs == 0) WoogleUtil::sendErrorMail("Timeout when executing job", "WoogleJob failed - adding new timeout" . ' / ' . $this->errorMsg);
				$this->insert();
				WoogleJobTimeout::addJob();
				return WoogleUtil::profileOut(__METHOD__, false);
			}
		} else {
			// configured timeout has not yet expired -> sleep again
			WoogleUtil::debug("Timeout has not yet expired - sleep again... max timeout is: " . $maxTimestamp . ", number of timeout jobs: " . $numJobs);
			$this->insert();
			WoogleJobTimeout::addJob($maxTimestamp); // use previous max timestamp
			return WoogleUtil::profileOut(__METHOD__, true);
		}
	}
	
	abstract protected function runJob();
}

?>
