<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

class WoogleSpecialConfig extends SpecialPage {
	
	private $returntitle;
	
	function __construct() {
		WoogleUtil::profileIn(__METHOD__);
		// set name of special page, permissions required to see it and if it should be listed in Specialpages
		parent::__construct(WoogleUtil::getSpecialPageLocal('Config'), 'userrights', true);
		WoogleUtil::profileOut(__METHOD__);
	}

	// Here the inline output of the Special page will be created
	function execute($par) {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		
		// show title
		$this->setHeaders();
		$this->returntitle = Title::makeTitle(NS_SPECIAL, WoogleUtil::getSpecialPageLocal('Config'));
		
		// check user permission
		if (!$this->userCanExecute($wgUser)) {
			$this->displayRestrictionError();
			return;
		}
		
		// check if call parameter was logfile
		if (strstr($par, '.log')){
			$lfile = WoogleUtil::getLogPath() . $par;
			if (file_exists($lfile)) $logfile = $lfile;
		}
		
		//
		//	Switch call parameter
		//
		
		if ($par == 'index_direct') {
			// index mode was chosen
			$this->executeIndexDirect();
			
		} else if ($par == 'index_jobs') {
			// index using jobs
			$this->executeIndexJobs();
			
		} else if ($par == 'initdb'){
			// init db was chosen
			$this->executeInitDb();
			
		} else if ($par == 'dropdb'){
			// dropping the db was chosen
			$this->executeDropDb();
	
		} else if (!empty($logfile)){
			// show logfile
			$this->executeShowLog($logfile);

		} else {
			// standard html output of config page
			$this->executeDefault();
		}

		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 *	standard html output of config page 
	 */
	private function executeDefault(){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		$html = 	"This is the page to configure the Woogle extension.";
		$html .=	"<br /><br />";
		
		
		// ----------------------------------------------------------------------------------------------------------------------
		// status section
		// ----------------------------------------------------------------------------------------------------------------------
		
		$html .= 	"<h2>Woogle system status</h2>";
		$native = WoogleUtil::getQueryService()->isNative();

		// -----------------------------------
		// status - index location
		// -----------------------------------
		
		// which kind of index service is used from where
		if ($native){
			$html .=	'Using local index at: ' . WoogleUtil::getIndexService()->getLocation() . '<br />';
		} else {
			$html .=	'Using remote index at: ' . WoogleUtil::getIndexService()->getLocation() . '<br />';
		}
		$html .=	"<br />";
		
		$html .=	'<table>';
		
		
		// -----------------------------------
		// status - index
		// -----------------------------------

		$html .=	'<tr><td>Index status</td><td>';
		try {
			$six =  WoogleUtil::getIndexService()->checkIndexService();
			if ($six){
				$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'ok.png"></img></td><td> - Everything ok.<br />';
			} else {
				$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - Directory not writable or connection problem.<br />';
			}
		} catch (Exception $e) {
			$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - Directory not writable or connection problem -' . $e->getMessage() . '<br />';
		}
		$html .=
			'</td><td><form name="reindex" action="' . htmlspecialchars($this->returntitle->getFullURL()) . '/index_direct" method="POST">' .
			'<input type="submit" value="(Re-)Create Index (Direct)"/> ' . '</form>' . '&nbsp;' .
			'<form name="reindex" action="' . htmlspecialchars($this->returntitle->getFullURL()) . '/index_jobs" method="POST">' .
			'<input type="submit" value="(Re-)Create Index (Jobs)"/> ' . '</form>'	. "\n";
		
		// PHP in safe_mode does not allow to change max_execution_time
		$safe_mode = ini_get('safe_mode');
		if ($safe_mode == True) {
			$html .=	'</td><td> Warning! PHP is running in safe mode. This prevents Woogle from changing the script exectution timeout limit. ';
			$html .= 	'You may face problems when trying to index many or large documents. In your php.ini set "safe_mode = off" manually.';
		}
		
		$html .=	'</td></tr>';

		// -----------------------------------
		// status - query service
		// -----------------------------------
		
		$html .=	'<tr><td>QueryService status</td><td>';
		try {
			$qix =  WoogleUtil::getQueryService()->checkQueryService();
			if ($qix > 0){
				$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'ok.png"></img></td><td> - Everything ok - ' . $qix . ' documents in index.<br />';
			} else {
				$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - No documents indexed or connection problem.<br />';
			}
		} catch (Exception $e) {
			$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - No documents indexed or connection problem.' . $e->getMessage() .'<br />';
		}
		$html .=	'</td><td>'; // no action required
		$html .=	'</td></tr>';
		
				
		// -----------------------------------
		// status - db logging
		// -----------------------------------

		if ($native){ // makes only sense for local index
			$html .=	'<tr><td>Database status</td><td>';

			
			// database tables exist
			if (WoogleMWUtil::tableExists('woogle_aggquery')){
				$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'ok.png"></img></td><td> - Everything ok.<br />';
				$can_drop = WoogleMWUtil::checkDbPrivileges('DROP');
				if ($can_drop){
					$html .=
						'</td><td><form name="dropdb" action="' . htmlspecialchars($this->returntitle->getFullURL()) . '/dropdb" method="POST">' .
						'<input type="submit" value="Remove database tables"/> ' .
						'</form>' . "\n";
				} else {
					$html .=	'</td><td>'; // no action required
				}
			// database tables not yet existing
			} else {
				$can_create = WoogleMWUtil::checkDbPrivileges('CREATE');
				if ($can_create){ // check permission
					$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - Logging tables do not exist yet.<br />';
					$html .=
						'</td><td><form name="createdb" action="' . htmlspecialchars($this->returntitle->getFullURL()) . '/initdb" method="POST">' .
						'<input type="submit" value="Create database tables"/> ' .
						'</form>' . "\n";
				} else {
					$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - Logging tables do not exist yet.<br />';
					$html .=	'</td><td>you need db root access to create tables'; // TODO										
					
				}
			}
			$html .=	'</td></tr>';
		}
		
		// -----------------------------------
		// status - log file
		// -----------------------------------

		$html .=	'<tr><td>Log file status</td><td>';
		$logpath = WoogleUtil::getLogPath();
		$writable = is_writable($logpath);
		if ($writable){
			$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'ok.png"></img></td><td> - Everything ok.<br />';
		} else {
			$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - Directory ' . $logpath .' is not writable.<br />';
		}
		$html .=	'</td><td>'; // no action required
		$html .=	'</td></tr>';
		
		// -----------------------------------
		// status - PCRE/UTF8 and mbstring
		// -----------------------------------
		
		// affects only Native because this is a Zend Lucene issue
		if ($native) {
			$html .=	'<tr><td>PCRE/UTF8 and mbstring</td><td>';

			$mbstringOkay = WoogleLucene::getMbstringSupport();
			$pcreOkay = WoogleLucene::getPcreSupport();

			$okay = ( $mbstringOkay and $pcreOkay );

			if ( $okay ) {
				$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'ok.png"></img></td><td> - Everything ok.<br />';
			} else {
					$html .= '<img width="20" height="20" src="' . WoogleUtil::getImagePath() . 'problem.png"></img></td><td> - ';
				if ( !$mbstringOkay ) {
					$html .= 'The PHP module mbstring is not loaded. ';			}
				if ( !$pcreOkay and !$mbstringOkay) {
					$html .= 'And ';
				}
				if ( !$pcreOkay ) {
					$html .= 'PCRE unicode support is not enabled on your system. ';
				}
				$html .= 'Woogle will not support unicode characters. <br />';

			}
			$html .=	'</td><td>';
			$html .=	'</td></tr>';
		}

		// end of status table
		$html .=	'</table>';
		$html .=	"<br /><br />";
		
		// ----------------------------------------------------------------------------------------------------------------------
		// configuration settings section
		// ----------------------------------------------------------------------------------------------------------------------
		
		$html .= 	"<h2>Woogle configuration settings</h2>";
		$html .=	'<table>';
		foreach (WoogleConfig::getKeys() as $configKey) {
			$html .= '<tr><td>' . $configKey . '</td><td>' . var_export(WoogleConfig::get($configKey), true) . '</td><td>' . WoogleConfig::getComment($configKey) . '</td></tr>';
		}
		$html .=	'</table><br /><br />';
		
		// ----------------------------------------------------------------------------------------------------------------------
		// error section (shows last 50 error log entries)
		// ----------------------------------------------------------------------------------------------------------------------
		
		$html .= 	"<h2>Woogle error log</h2>";
		$error_string = WoogleUtil::getErrors();
		if ($error_string != ""){
			$html .=    $error_string . "<br />";				
		} else {
			$html .=    "<br />Everything fine<br />";
		}

		$html .=	"<br /><br />";
		$wgOut->addHTML($html);
	
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	
	
	/**
	 *	Re-Index all content directly (in this HTTP session) 
	 */
	private function executeIndexDirect(){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		set_time_limit(900); //set timeout limit to 15 minutes (for both Native and Remote)
		WoogleUtil::getIndexService()->index(true); // will create $wgOut output
		$wgOut->addHTML('<p>return to <a href="' . htmlspecialchars($this->returntitle->getFullURL()) . '">'. WoogleUtil::getSpecialPageLocal('Config') .'</a></p>');
					
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 *	Re-Index all content using MediaWiki jobs 
	 */
	private function executeIndexJobs(){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		set_time_limit(900); //set timeout limit to 15 minutes (for both Native and Remote)
		WoogleUtil::getIndexService()->index(false); // will create $wgOut output
		$wgOut->addHTML('<p>return to <a href="' . htmlspecialchars($this->returntitle->getFullURL()) . '">'. WoogleUtil::getSpecialPageLocal('Config') .'</a></p>');
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 *	Create Woogle-specific databases 
	 */
	private function executeInitDb(){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		// check if table not exists
		if (!WoogleMWUtil::tableExists('woogle_aggquery')){
			try {
				$count = WoogleQueryManagerNative::initDb();
				$wgOut->addHTML('Created ' . $count . ' tables in the database.');
			
			} catch (Exception $e) {
				$wgOut->addHTML('Database error - check your access rights: ' . $e);
			}
			
			$wgOut->addHTML('<p>return to <a href="' . htmlspecialchars($this->returntitle->getFullURL()) . '">'. WoogleUtil::getSpecialPageLocal('Config') .'</a></p>');
		} else {
			$wgOut->addHTML('Error - table woogle_aggquery already exists!');
		}
	
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 *	Remove Woogle-specific databases 
	 */
	private function executeDropDb(){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		// check if table exists
		if (WoogleMWUtil::tableExists('woogle_aggquery')){
			try {
				$count = WoogleQueryManagerNative::dropDb();
				$wgOut->addHTML('Removed ' . $count . ' tables from the database.');
			
			} catch (Exception $e) {
				$wgOut->addHTML('Database error - check your access rights: ' . $e);
			}
			
			$wgOut->addHTML('<p>return to <a href="' . htmlspecialchars($this->returntitle->getFullURL()) . '">'. WoogleUtil::getSpecialPageLocal('Config') .'</a></p>');
		} else {
			$wgOut->addHTML('Error - table woogle_aggquery does not exist!');
		}
		
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Pass through logfile
	 * 
	 * @param $logfile		String, logfile name
	 * @return				void/exit
	 */
	private function executeShowLog($logfile){
		global $wgOut;
		WoogleUtil::profileIn(__METHOD__);
		
		$xFile = fopen($logfile, 'r');
		fpassthru($xFile);
		fclose($xFile);
		WoogleUtil::profileOut(__METHOD__);
		exit;
	}
	
	

}

?>
