<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

require_once('blankstart.php');

WoogleUtil::profileIn('opensearch.php');

global $wgSitename, $wgRequest, $wgEnableApi;

$type = $_GET['type']; // 'wiki' or 'all' 

$shortTitle	= 'WoogleWiki';
$longTitle 	= WoogleUtil::getMsg('OpenSearchLongTitleDefault', $wgSitename);
$desc 	= WoogleUtil::getMsg('OpenSearchDescDefault', $wgSitename);
$repo 	= '&amp;repositories[]=' . WoogleConfig::get('indexRepository');

if ($type == 'all'){
	$shortTitle	= 'WoogleAll';
	$longTitle 	= WoogleUtil::getMsg('OpenSearchLongTitleAll', $wgSitename);
	$desc 	= WoogleUtil::getMsg('OpenSearchDescAll');
	$repo 	= '';
}

if( $wgRequest->getVal( 'ctype' ) == 'application/xml' ) {
	// Makes testing tweaks about a billion times easier
	$ctype = 'application/xml';
} else {
	$ctype = 'application/opensearchdescription+xml';
}

$response = $wgRequest->response();
$response->header( "Content-type: $ctype" );

// Set an Expires header so that squid can cache it for a short time
// Short enough so that the sysadmin barely notices when $wgSitename is changed
$expiryTime = 600; # 10 minutes
$response->header( 'Expires: ' . gmdate( 'D, d M Y H:i:s', time() + $expiryTime ) . ' GMT' );
$response->header( 'Cache-control: max-age=600' );

$xml = '<?xml version="1.0"?>';
$xml .= Xml::openElement( 'OpenSearchDescription',
	array(
		'xmlns' => 'http://a9.com/-/spec/opensearch/1.1/',
		'xmlns:moz' => 'http://www.mozilla.org/2006/browser/search/' ) 
	);

// Set meta-informations
$xml .= Xml::element( 'ShortName', null, $shortTitle );
$xml .= Xml::element( 'LongName', null, $longTitle );
$xml .= Xml::element( 'Description', null, $desc );
$xml .= Xml::element( 'InputEncoding', null, 'UTF-8');
$xml .= Xml::element( 'OutputEncoding', null, 'UTF-8');
$xml .= Xml::element( 'AdultContent' , null, 'false');
$xml .= Xml::element( 'Language' , null, 'en-us');
$xml .= Xml::element( 'Developer', null , 'FZI');
$xml .= Xml::element( 'Contact' , null, 'info@teamweaver.org' );
$xml .= Xml::element( 'Tags' , null, 'Woogle' );
$attributString = 'Search on ' . WoogleMWUtil::getWikiAbsolutePath(); 
$xml .= Xml::element( 'Attribution', null, $attributString);

// By default we'll use the site favicon.
// Double-check if IE supports this properly?
$imgPath = WoogleUtil::getWoogleAbsolutePath() . 'resources/img/favicon.ico';
$xml .= Xml::element( 'Image',
	array(
		'height' => 16,
		'width' => 16,
		'type' => 'image/x-icon' ),
	$imgPath );

$urls = array();

// General search template. Given an input term, this should bring up
// search results or a specific found page.
// At least Firefox and IE 7 support this.
$searchPage = WoogleMWUtil::getAbsolutePath() . WoogleUtil::getWoogleLinkPrefix() . '{searchTerms}?ref=browser' . $repo ;
$urls[] = array(
	'type' => 'text/html',
	'method' => 'get',
	'template' => $searchPage );

if( $wgEnableAPI ) {
	// JSON interface for search suggestions.
	// Supported in Firefox 2 and later.
	$extSearchPage = WoogleUtil::getWoogleAbsolutePath() . 'services/suggest.php?q={searchTerms}';
	$urls[] = array(
		'type' => 'application/x-suggestions+json',
		'method' => 'get',
		'template' => $extSearchPage );
}

// Allow hooks to override the suggestion URL settings in a more
// general way than overriding the whole search engine...
wfRunHooks( 'OpenSearchUrls', array( &$urls ) );

foreach( $urls as $attribs ) {
	$xml .= Xml::element( 'Url', $attribs );
}

// And for good measure, add a link to the straight search form.
// This is a custom format extension for Firefox, which otherwise
// sends you to the domain root if you hit "enter" with an empty
// search box.
$xml .= Xml::element( 'moz:SearchForm', null, WoogleMWUtil::getAbsolutePath().WoogleUtil::getWoogleSearchSpecialPageName());

$xml .= '</OpenSearchDescription>';

print $xml;

WoogleUtil::profileOut('opensearch.php');
?>
